*==============================================================================
* Function:			NaturalSort
* Purpose:			Sorts an array in natural rather than ASCII order
* Author:			Doug Hennig
* Last Revision:	08/09/2017
* Parameters:		taArray      - the array to sort (passed by reference)
*					tnColumn     - the column to sort on (optional: if it isn't
*						specified, column 1 is used)
*					tlDescending - .T. to sort in descending order; if .F. or
*						not specified, ascending order is used
* Returns:			.T. if it succeeded or .F. if invalid parameters are passed
*						or the specified column doesn't contain a homogeneous
*						data type
* Environment in:	none
* Environment out:	none
*==============================================================================

lparameters taArray, ;
	tnColumn, ;
	tlDescending
local lnColumn, ;
	lnRows, ;
	lnCols, ;
	lnOrder, ;
	laArray[1], ;
	lnI, ;
	lcKey, ;
	lcString, ;
	laClone[1], ;
	lnIndex

* Ensure taArray is an array and tlDescending is logical if it's specified.

if type('taArray', 1) <> 'A' or ;
	(pcount() = 3 and vartype(tlDescending) <> 'L')
	return .F.
endif type('taArray', 1) <> 'A' ...

* If the column to sort on wasn't specified, assume 1.

lnColumn = iif(pcount() = 2, tnColumn, 1)

* Figure out the size of the source array.

lnRows = alen(taArray, 1)
lnCols = alen(taArray, 2)

* Ensure the column to sort on is valid.

if vartype(lnColumn) <> 'N' or not between(lnColumn, 1, max(lnCols, 1))
	return .F.
endif vartype(lnColumn) <> 'N' ...

* Figure out the order flag for ASORT().

lnOrder = iif(tlDescending, 1, 0)

* Get the data type of the first key value. If it isn't character, we don't
* have to do anything fancy; ASORT() will take care of it for us.

if vartype(taArray[1, lnColumn]) = 'C'

* Create an array we'll sort on.

	dimension laArray[lnRows, 2]

* Go through each element we're sorting on, get its natural sort key, and store
* the key and the original index in our sort array.

	for lnI = 1 to lnRows
		if lnCols = 0
			lcKey = taArray[lnI]
		else
			lcKey = taArray[lnI, lnColumn]
		endif lnCols = 0
		lcString = NaturalSortKey(lcKey)
		if not isnull(lcString)
			laArray[lnI, 1] = lcString
			laArray[lnI, 2] = lnI
		else
			return .F.
		endif not isnull(lcString)
	next lnI

* Now sort the array and reorder the values in the source array by cloning it
* and copying the values from the each row in the clone to the new row in the
* source array.

	asort(laArray, 1, -1, lnOrder, 1)
	acopy(taArray, laClone)
	for lnI = 1 to lnRows
		lnIndex = laArray[lnI, 2]
		if lnCols > 0
			for lnJ = 1 to lnCols
				taArray[lnI, lnJ] = laClone[lnIndex, lnJ]
			next lnJ
		else
			taArray[lnI] = laClone[lnIndex]
		endif lnCols > 0
	next lnI

* Just use ASORT to do the job.

else
	asort(taArray, lnColumn, -1, lnOrder, 0)
endif vartype(taArray[1, lnColumn]) = 'C'
return .T.
