*==============================================================================
* Function:			NaturalSortKey
* Purpose:			Creates a key suitable for natural sorting from the
*						specified key
* Author:			Doug Hennig
* Last Revision:	08/09/2017
* Parameters:		tcKey    - the original key
*					tnLength - the length to pad numeric sections to (optional:
*						if it isn't specified, 20 is used)
* Returns:			the key formatted for natural sorting (numeric sections
*						left padded with zeros to a consistent length) or null
*						if either parameter is invalid
* Environment in:	none
* Environment out:	none
*==============================================================================

lparameters tcKey, ;
	tnLength
local lnLength, ;
	llInNumeric, ;
	lcString, ;
	lnI, ;
	lcChar, ;
	llNumeric, ;
	lcNumeric

* Define the constants.

#define cnLENGTH                   20
	&& the length to pad numeric sections to by default

* Bug out if the data type is wrong.

if vartype(tcKey) <> 'C'
	return .NULL.
endif vartype(tcKey) <> 'C'

* Use a default length if not specified.

do case
	case pcount() = 1
		lnLength = cnLENGTH
	case vartype(tnLength) = 'N' and between(tnLength, 1, 60)
		lnLength = tnLength
	otherwise
		return .NULL.
endcase		

* Create a key that will sort properly by looking for numeric sections and
* left-padding them with zeros.

llInNumeric = .F.
lcString    = ''
for lnI = 1 to len(tcKey)
	lcChar    = substr(tcKey, lnI, 1)
	llNumeric = isdigit(lcChar) or ;
		(lcChar = '.' and isdigit(substr(tcKey, lnI + 1, 1)))
	do case
		case llNumeric and llInNumeric
			&& if we have a digit and we're already in a numeric
			&& section, add to the numeric part
			lcNumeric = lcNumeric + lcChar
		case llNumeric
			&& if we have a digit and we're not in a numeric
			&& section, flag that we are in such a section and add to
			&& the numeric part
			llInNumeric = .T.
			lcNumeric   = lcChar
		case llInNumeric
			&& we don't have a digit and we were in a numeric section
			&& so pad the section and add it to our string
			llInNumeric = .F.
			lcString    = lcString + padl(lcNumeric, lnLength, '0') + ;
				lcChar
		otherwise
			lcString = lcString + lcChar
	endcase
next lnI

* Finish the string if we were still processing a numeric section.

if llInNumeric
	lcString = lcString + padl(lcNumeric, lnLength, '0')
endif llInNumeric
return lcString
